const { KIND_APP } = require('../module');
const { App } = require('../app');
const assert = require('assert');

class MockApp extends App {
  static createMessage (address, command, payload) {
    assert(typeof payload !== 'object' || payload instanceof Buffer, 'Payload must be buffer or primitive type');

    return {
      address,
      kind: KIND_APP,
      module: 'mock',
      command,
      payload,
    };
  }

  constructor () {
    super('mock', require('../package.json').version);

    this.messages = [];
    this.waitingList = [];
    this.requestHandlers = {};
  }

  up () {
    this.messages = [];
    this.waitingList = [];
    this.requestHandlers = {};
  }

  down () {
    this.waitingList.forEach(wait => {
      clearTimeout(wait.timeout);
      wait.reject(new Error('Mock application down, cleaning waiting list'));
    });

    this.messages = [];
    this.waitingList = [];
    this.requestHandlers = {};
  }

  onMessage (message) {
    let newWaitingList = [];
    this.waitingList.forEach(wait => {
      if (wait.message.command === message.command && this._equalPayload(wait.message.payload, message.payload)) {
        clearTimeout(wait.timeout);
        wait.resolve();
      } else {
        newWaitingList.push(wait);
      }
    });
    this.waitingList = newWaitingList;

    this.messages.push(message);
  }

  onRequest ({ address, command, payload }) {
    let handler = this.requestHandlers[command];
    if (handler) {
      return handler({ address, command, payload });
    }
  }

  _equalPayload (payload1, payload2) {
    if (payload1.compare) {
      return payload1.compare(payload2) === 0;
    }

    return payload1 === payload2;
  }

  gotMessage ({ command, payload }) {
    return this.messages.filter(message => {
      return message.command === command && this._equalPayload(message.payload, payload);
    });
  }

  async waitForMessage (message, tTimeout = 1000) {
    if (this.gotMessage(message).length) {
      return;
    }

    await new Promise((resolve, reject) => {
      let timeout = setTimeout(() => {
        clearTimeout(timeout);
        let index = this.waitingList.indexOf(wait);
        if (index !== -1) {
          this.waitingList.splice(index, 1);
        }

        reject(new Error('Mock timeout while waiting for message'));
      }, tTimeout);
      let wait = { resolve, reject, timeout, message };
      this.waitingList.push(wait);
    });
  }

  setRequestHandler (command, handler) {
    this.requestHandlers[command] = handler;
  }
}

module.exports = MockApp;
