const net = require('net');
const { URL } = require('url');
const { Channel } = require('../channel');
const getIps = require('../lib/get-ips');
const { hidden } = require('../lib/prop');

const LISTENING_TIMEOUT = 3000;

class Tcp extends Channel {
  constructor ({ port, host = '0.0.0.0', listening = true } = {}) {
    super('tcp', require('../package.json').version);

    this.listening = listening;
    this.host = host;
    this.port = port;

    hidden(this, 'server');
  }

  async up () {
    if (!this.listening) {
      return;
    }

    this.server = net.createServer(socket => this.incoming(socket));

    await new Promise((resolve, reject) => {
      let timeout = setTimeout(() => reject(new Error('Timeout on listening')), LISTENING_TIMEOUT);
      this.server.listen(this.port, this.host, () => {
        clearTimeout(timeout);
        this.port = this.server.address().port;
        resolve();
      });
    });

    this.logger.log(`Listening at ${this.host}:${this.port}`);
  }

  onMessage () {
    // noop
  }

  async connect (url) {
    let u = new URL(url);
    let port = u.port;
    let host = u.hostname;

    let socket = await new Promise((resolve, reject) => {
      let socket = net.connect(port, host, () => resolve(socket));
      socket.on('error', err => reject(err));
    });

    return socket;
  }

  async down () {
    if (!this.server) {
      return;
    }

    await new Promise(resolve => {
      this.server.close(resolve);
      this.server = null;
    });
  }

  getUrls () {
    if (!this.listening) return [];

    let ips = getIps();
    return ips.map(ip => `${this.proto}://${ip}:${this.port}`);
  }
}

module.exports = Tcp;
