const repl = require('repl');
const { Daemon } = require('../../daemon');
const { Config } = require('twlv-config');
const path = require('path');
const { Swarm } = require('../../swarm');
const FileStorage = require('twlv-storage-file');
const logger = require('twlv-logger')('twlv-swarm:cli:cmds:console');

// TODO: revisit this later
module.exports = {
  command: 'console',
  desc: 'Run console',

  async handler (argv) {
    let dataDir = argv.dataDir;
    let configFile = path.join(dataDir, 'twlvd.json');

    let config = new Config({ dataDir, useLocking: true });
    await config.load(configFile);
    config.set('dataDir', dataDir);
    await config.persist();

    let swarm = new Swarm({ storage: new FileStorage({ dataDir }) });
    let daemon = new Daemon(config, swarm);
    ['SIGTERM', 'SIGINT', 'SIGBREAK', 'SIGUSR2'].forEach((signal) => {
      process.once(signal, async () => {
        logger.log('Caught signal %o', signal);
        await daemon.stop();
        process.kill(process.pid, signal);
      });
    });

    try {
      await daemon.start();
    } catch (err) {
      logger.error('Failed to start daemon,', err);
      process.exit(1);
    }

    let console = repl.start('> ');
    console.context.daemon = daemon;
  },
};
