const { EventEmitter } = require('events');
const { readonly } = require('../../lib/prop');
const { KIND_APP, KIND_CHANNEL, KIND_DISCOVERY } = require('../../module');

class RelaySession extends EventEmitter {
  constructor (swarm, peer) {
    super();

    this.peer = peer;
    readonly(this, 'swarm', swarm);
  }

  get address () {
    return this.peer ? this.peer.address : undefined;
  }

  get identity () {
    return this.peer ? this.peer.identity : undefined;
  }

  end () {

  }

  async write ({ mode, kind, module, command, payload }) {
    await this.swarm.relay({
      address: this.address,
      kind,
      module,
      command,
      payload,
    });
  }

  advertise (kind, module) {
    if (!this.peer) {
      return false;
    }

    switch (kind) {
      case KIND_APP:
        return this.peer.apps.indexOf(module) !== -1;
      case KIND_CHANNEL:
        return this.peer.channels.indexOf(module) !== -1;
      case KIND_DISCOVERY:
        return this.peer.discoveries.indexOf(module) !== -1;
    }
  }
}

module.exports = { RelaySession };
