const { Peer } = require('../../peer');
const { Logger } = require('twlv-logger');
const { RelaySession } = require('./relay-session');
const { KIND, MODULE, CMD_ADV, CMD_ACK } = require('./constants');

class HandshakeRelayUnit {
  constructor ({ swarm, timeout, address, initiate }) {
    this.address = address;
    this.swarm = swarm;
    this.timeout = timeout;
    this.initiate = initiate;
    this.logger = new Logger('twlv-swarm:controller:handshake:relay-unit', swarm.address.substr(0, 4));
  }

  async createSession (advertisement) {
    await this.swarm.relay({
      address: this.address,
      kind: KIND,
      module: MODULE,
      command: CMD_ADV,
      payload: this.swarm.advertisement,
    });

    let session = await new Promise((resolve, reject) => {
      this._timeout = setTimeout(() => {
        this._resolveCallback = undefined;
        this._rejectCallback = undefined;
        reject(new Error('Handshake timeout'));
      }, this.timeout);

      this._resolveCallback = resolve;
      this._rejectCallback = reject;

      if (advertisement) {
        this._onAdvMessage(advertisement);
      }
    });

    return session;
  }

  _onMessage ({ kind, module, command, payload }) {
    if (kind !== KIND || module !== MODULE) {
      return;
    }

    if (command === CMD_ADV) {
      this._onAdvMessage(payload);
    } else if (command === CMD_ACK) {
      this._onAckMessage();
    }
  }

  _onAdvMessage (payload) {
    this.peer = new Peer(payload);
    this.swarm.relay({
      address: this.address,
      kind: KIND,
      module: MODULE,
      command: CMD_ACK,
    });

    this._acking = true;
    this._checkAcks();
  }

  _onAckMessage () {
    this._acked = true;
    this._checkAcks();
  }

  _checkAcks () {
    if (!this._acking || !this._acked) {
      return;
    }

    let session = new RelaySession(this.swarm, this.peer);

    let resolve = this._resolveCallback;
    clearTimeout(this._timeout);
    this._resolveCallback = undefined;
    this._rejectCallback = undefined;

    resolve(session);
  }
}

module.exports = { HandshakeRelayUnit };
