const { Discovery } = require('../discovery');
const { Peer } = require('../peer');
const CMD_QUESTION = 'q';
const CMD_ANSWER = 'a';

class PeerLookup extends Discovery {
  constructor () {
    super('peer-lookup', require('../package.json').version);
  }

  up () {
    this.cache = {};
  }

  down () {
    for (let address in this.cache) {
      let entry = this.cache[address];
      entry.callbacks.forEach(({ resolve }) => {
        resolve(entry.definition);
      });
    }

    this.cache = {};
  }

  async onMessage ({ address, command, payload }) {
    // this.logger.log('onMessage %o', { address, command, payload });
    switch (command) {
      case CMD_QUESTION:
        let queryAddress = payload.toString('hex');

        // see if swarm already have peer definition with address
        let definition = await this.lookup(queryAddress);

        // when no peer definition found, halt
        if (!definition) {
          return;
        }

        // when found peer definition, and there is outstanding lookup halted
        if (queryAddress in this.cache) {
          let cache = this.cache[queryAddress];
          cache.definition = definition;
          let callback;
          while ((callback = cache.callbacks.shift())) {
            callback.resolve(definition);
          }
        }

        this.send({ address, command: CMD_ANSWER, payload: definition });
        break;
      case CMD_ANSWER: {
        let queryAddress = payload.address;
        if (queryAddress in this.cache) {
          let cache = this.cache[queryAddress];

          let definition = new Peer(payload);

          cache.definition = definition;

          let callback;
          while ((callback = cache.callbacks.shift())) {
            callback.resolve(definition);
          }
        }
        break;
      }
    }
  }

  async onLookup (address) {
    this.logger.log('onLookup %s', address);
    if (address in this.cache === false) {
      this.cache[address] = { callbacks: [] };
      this.broadcast({ command: CMD_QUESTION, payload: Buffer.from(address, 'hex') });
    }

    let cache = this.cache[address];
    if (!cache.definition) {
      let result = await new Promise((resolve, reject) => cache.callbacks.push({ resolve, reject }));
      return result;
    } else {
      return cache.definition;
    }
  }
}

module.exports = PeerLookup;
