const assert = require('assert');
const { Identity } = require('twlv-identity');
const { hidden } = require('./lib/prop');

let ID = 0;

class Peer {
  constructor ({
    address = '',
    urls = [],
    apps = [],
    channels = [],
    discoveries = [],
    publicKey,
    timestamp = new Date(),
  }) {
    assert(address, 'Peer must define address');

    this.id = ID++;
    this.address = address;
    this.urls = urls;
    this.apps = apps;
    this.channels = channels;
    this.discoveries = discoveries;
    this.timestamp = new Date(timestamp);

    let identity;
    if (publicKey) {
      identity = new Identity(publicKey);
      if (identity.address !== address) {
        throw new Error('Session is not coming from valid address');
      }
    }
    hidden(this, 'identity');
  }

  update (peer) {
    assert.ok(peer instanceof Peer, 'Peer to update must be instanceof peer');
    assert.equal(peer.address, this.address, 'Cannot update peer of different address');

    if (peer.timestamp > this.timestamp) {
      this.urls = peer.urls;
      this.apps = peer.apps;
      this.channels = peer.channels;
      this.discoveries = peer.discoveries;
      this.timestamp = peer.timestamp;
      this.identity = peer.identity;
    }
  }

  toString () {
    return `peer:${this.address}`;
  }
}

module.exports = { Peer };
